#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <errno.h>


#include "config.h"
#include "q2-wrapper.h"


int	quit_verbosity;

/*struct directories dir; */

int main (int argc, char *argv[])
{
	int	myuid;
	int	myeuid;
	int	option;
	int	childpid;
	char	lineout[LINELEN+1];
	char	binpath[PATHLEN+1];

	atexit(die);

	myuid = getuid();
	myeuid = geteuid();

#ifndef	_DEBUG_
	if (seteuid(myuid) != 0) {
		perror("\nError dropping root privileges!\n");
		quit_verbosity = TERSE_QUIT;
		exit(1);
	}
#endif

	while ((option = getopt(argc, argv, "qhv")) != EOF) {
		switch (option) {
		case 'q':
			printf("-q option unimplemented.\n");
			break;
		case 'v':
			printf("-v option unimplemented.\n");
			break;
		case 'h':
			usage(argv[0]);
			break;
		default:
			usage(argv[0]);
		}
	}

#ifndef _DEBUG_
	if (myeuid != 0) {
		fprintf(stderr, "%s must be run suid root.\n", argv[0]);
		fprintf(stderr, "Read the README.\n\n");
		quit_verbosity = TERSE_QUIT;
		exit(1);
	}
#endif

	makeline(lineout, argc, argv);	/* Make the command line */

	if (!getpaths()) {
		fprintf(stderr, "$HOME environmental variable doesn't exist.");
		fprintf(stderr, "This shouldn't happen.\n");
		exit(1);
	}


#ifdef	_DEBUG_
	printf("%s\n", argv[0]);
	printf("\n%s\n\n", lineout);
	printf("uid = %d : euid = %d\n", getuid(), geteuid());
	printf("---struct directories---\n");
	printf("home    %s\n", dir.home);
	printf("dot     %s\n", dir.dot);
	printf("save    %s\n", dir.save);
	printf("pic     %s\n", dir.pic);
	printf("demo    %s\n", dir.demo);
	printf("players %s\n", dir.players);
	printf("video   %s\n\n", dir.video);
#endif


	greetings();
	quit_verbosity = SMALL_QUIT;

	printf("------- Setting up directories -------\n");
	printf("Checking $HOME/%s\n", config.user_dir);
	if (!(checkdirs())) {
		fprintf(stderr, "Something's wrong with the directories\n"); 
	}

	printf("\n");
	quit_verbosity = BIG_QUIT;

	strncpy(binpath, config.binpath, PATHLEN);

	if (seteuid(0) != 0) {
		perror("\nError regaining needed root privileges!\n");
		exit(1);
	}

	printf("Symlinking...\n");
	if (!makelinks()) {
		fprintf(stderr, "Unable to symlink from $HOME/%s to %s.\n", 
					config.user_dir, config.sys_baseq2);
		exit(1);
	}

	switch (childpid = fork()) {
	case 0:		/* child */
		printf("------- Executing Quake2 -------\n");
		if (execl(binpath, lineout) != 0) {
			perror("\nError executing the Quake2 binary!\n");
			exit(1);
		}
		break;
	case -1:	/* forkbarf */
		perror("Fork problem.  Can't execute Quake2\n");
		exit(1);
		break;
	default:	/* parent */
	/* At this point, we should monitor dir.pic for changes. */
	/* That would indicate that a screendump was just made and that */
	/* we should do some sort of conversion on the dump file. */
	/* Thankfully, Quake2 properly sets the permissions on things */
	/* like saves.  Otherwise we'd have to monitor everything in the */
	/* directory structure for changes and that would introduce a race */
	/* condition of some sort.  The impact?  I don't know. */
		
	/* screendump conversion unimplemented */
		break;

	}
/*
	if (waitpid(childpid, 0, NULL) != childpid) {
		perror("Something went wrong waiting for Quake2 to finish.\n");
		exit(1);
	}
*/

/*
	if (wait(NULL) != childpid) {
		perror("Oops.\n");
		exit(1);
	}	
*/
wait(NULL);

	printf("q2-wrapper back in control.\n");

#ifndef _DEBUG_
	if (seteuid(myuid) != 0) {
		perror("\nError dropping root privileges!\n");
		exit(1);
	}
#endif

	printf("Tearing down symlinks...\n");
	if (!breaklinks()) {
		fprintf(stderr, "Unable to remove symlink from $HOME/%s to %s.\n", 
			config.user_dir, config.sys_baseq2);
		exit(1);
	}
	
	printf("======= q2-wrapper terminated normally =======\n");

	return(0);
}

